"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const failure_1 = require("./failure");
const success_1 = require("./success");
const equality_1 = require("../equality");
(0, vitest_1.describe)("Failure Tests", () => {
    (0, vitest_1.test)("getでthrowされる", () => {
        const either = (0, failure_1.failure)(new Error("error"));
        (0, vitest_1.expect)(() => either.get).toThrowError("failure to get");
    });
    (0, vitest_1.test)("getErrorで値を返す", () => {
        const either = (0, failure_1.failure)(new Error("error"));
        (0, vitest_1.expect)(either.getError).toEqual(new Error("error"));
    });
    (0, vitest_1.test)("mapで何も起こらない", () => {
        const either = (0, failure_1.failure)(new Error("error"));
        const expected = (0, failure_1.failure)(new Error("error"));
        const actual = either.map((n) => `${n}番`);
        (0, vitest_1.expect)(actual).toEqual(expected);
    });
    (0, vitest_1.test)("flatMapで何も起こらない", () => {
        const either = (0, failure_1.failure)(new Error("error"));
        const expected = (0, failure_1.failure)(new Error("error"));
        const actual = either.flatMap((n) => (0, success_1.success)(`${n}番`));
        (0, vitest_1.expect)(actual).toEqual(expected);
    });
    (0, vitest_1.test)("mapErrorで値を変換できる", () => {
        const either = (0, failure_1.failure)(new Error("error"));
        const expected = (0, failure_1.failure)(new Error("error+mapError"));
        const actual = either.mapError((error) => new Error(error.message + "+mapError"));
        (0, vitest_1.expect)(actual).toEqual(expected);
    });
    (0, vitest_1.test)("flatMapErrorで値を変換し、平らにできる", () => {
        const either = (0, failure_1.failure)(new Error("error"));
        const expected = (0, failure_1.failure)(new Error("error+mapError"));
        const actual = either.flatMapError((error) => (0, failure_1.failure)(new Error(error.message + "+mapError")));
        (0, vitest_1.expect)(actual).toEqual(expected);
    });
    (0, vitest_1.test)("isSuccessでfalseを返し、isFailureでtrueを返す", () => {
        const either = (0, failure_1.failure)(new Error("error"));
        (0, vitest_1.expect)(either.isSuccess).toBeFalsy();
        (0, vitest_1.expect)(either.isFailure).toBeTruthy();
    });
    (0, vitest_1.test)("equalsで同じ値ならばtrueを返す", () => {
        const either1 = (0, failure_1.failure)(10);
        const either2 = (0, failure_1.failure)(10);
        (0, vitest_1.expect)(either1.equals(either2)).toBeTruthy();
    });
    (0, vitest_1.test)("equalsで異なる値ならばfalseを返す", () => {
        const either1 = (0, failure_1.failure)(10);
        const either2 = (0, failure_1.failure)(20);
        (0, vitest_1.expect)(either1.equals(either2)).toBeFalsy();
    });
    (0, vitest_1.test)("equalsでequatableならばequalsで比較する", () => {
        class E extends equality_1.Equatable {
            constructor(n) {
                super();
                this.n = n;
            }
        }
        const either1 = (0, failure_1.failure)(new E(10));
        const either2 = (0, failure_1.failure)(new E(10));
        (0, vitest_1.expect)(either1.equals(either2)).toBeTruthy();
    });
    (0, vitest_1.test)("successと比較するとfalseを返す", () => {
        const either1 = (0, failure_1.failure)(10);
        const either2 = (0, success_1.success)(10);
        (0, vitest_1.expect)(either1.equals(either2)).toBeFalsy();
    });
    (0, vitest_1.test)("equalsで第三引数に比較関数を渡すと、その関数で比較する", () => {
        const either1 = (0, failure_1.failure)(10);
        const either2 = (0, failure_1.failure)(20);
        (0, vitest_1.expect)(either1.equals(either2, () => true, (a, b) => a % 10 === b % 10)).toBeTruthy();
    });
    (0, vitest_1.test)("equalsで第三引数に比較関数を渡すと、その関数がfalseを返すならfalseを返す", () => {
        const either1 = (0, failure_1.failure)(10);
        const either2 = (0, failure_1.failure)(20);
        (0, vitest_1.expect)(either1.equals(either2, () => true, () => false)).toBeFalsy();
    });
    (0, vitest_1.test)("toOptionalでnoneを返す", () => {
        const either = (0, failure_1.failure)(10);
        (0, vitest_1.expect)(either.toOptional.isEmpty).toBeTruthy();
    });
});
//# sourceMappingURL=failure.test.js.map