"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.some = exports.Some = void 0;
const __1 = require("../");
const equality_1 = require("../equality");
const none_1 = require("./none");
const clone_1 = require("../clone");
const compare_1 = require("../compare");
const codable_1 = require("../codable");
const ts_md5_1 = require("ts-md5");
const hash_1 = require("../hash");
class Some {
    constructor(value) {
        this.value = value;
    }
    get get() {
        return this.value;
    }
    getOrElse(defaultValue) {
        return this.value;
    }
    orElse(defaultValue) {
        return this;
    }
    isDefined() {
        return true;
    }
    isEmpty() {
        return false;
    }
    map(f) {
        return new Some(f(this.value));
    }
    flatMap(f) {
        return f(this.value);
    }
    equals(compared, comparisonFunc) {
        if (compared instanceof none_1.None)
            return false;
        if (!(compared instanceof Some)) {
            return (0, equality_1.equals)(this.value, compared);
        }
        if (comparisonFunc === undefined) {
            return (0, equality_1.equals)(this.value, compared.value);
        }
        return comparisonFunc(this.value, compared.value);
    }
    fold(left, right) {
        return right(this.value);
    }
    unwrap(_) {
        return this.value;
    }
    toEither(_) {
        return (0, __1.success)(this.value);
    }
    get clone() {
        if ((0, clone_1.isClonable)(this.value)) {
            return some(this.value.clone);
        }
        if (typeof this.value === "object") {
            return some(Object.create(this.value));
        }
        return some(this.value);
    }
    get json() {
        if ((0, codable_1.isJSONEncodable)(this.value)) {
            return this.value.json;
        }
        return this.value;
    }
    compare(compared) {
        return compared.map((value) => (0, compare_1.compare)(this.value, value)).getOrElse(true);
    }
    get hashValue() {
        return ts_md5_1.Md5.hashStr("some" + (0, hash_1.hash)(this.value));
    }
}
exports.Some = Some;
function some(value) {
    return new Some(value);
}
exports.some = some;
//# sourceMappingURL=some.js.map